 /******************** Plugin info **********************\
 *                                                       *
 * Name: plugin_bm_rindy_player                          *
 * Author: Rinde (rinde@fiatnox.de)                      *
 * Version: 1.0 RC1                                      *
 * Released: 2004/04/05                                  *
 *                                                       *
 * Requirements:                                         *
 *  o  Half-Life dedicated or listen-server              *
 *  o  Admin-Mod version: 2.50.50                        *
 *  o  plugin_base_menu installed                        *
 *  o  Include files:                                    *
 *      -  menu.inc                                      *
 *      -  rindy.inc                                     *
 *      -  checksetting.inc                              *
 *      -  rindy_player_helper.inc                       *
 *      -  rindy_player_infos.inc                        *
 *      -  rindy_player_menus.inc                        *
 *  o  Settings:                                         *
 *      -  amv_enable_beta "menu1"                       *
 *      -  file_access_read "1"                          *
 *      -  file_access_write "1"                         *
 *      -  A valid vault-file                            *
 *  o  Configuration files:                              *
 *      -  admins.ini                                    *
 *      -  reasons.ini                                   *
 *                                                       *
 ********************* Description ***********************
 *                                                       *
 * Features:                                             *
 *  o A comfortable menu to choose player and punishment *
 *  o Several grades of punishment, including warn,      *
 *    slap, slay, kick and ban                           *
 *  o Allows to give a reason with every punishment      *
 *  o Reasons are selectable through menues as well      *
 *  o Logging of usage into a seperate file              *
 *  o Own access right management to allow more flexible *
 *    rights                                             *
 *  o Two levels of admin immunity                       *
 *  o Visualization of the userstatus in menu            *
 *  o Stores data of players after they left the server  *
 *  o Banning players after they left by menu            *
 *  o Notification if some previously warned/kicked      *
 *    player rejoins with a different nickname           *
 *  o Support for Steam and LAN servers                  *
 *                                                       *
 * Installation:                                         *
 *  1. Place the files "plugin_bm_rindy_player.sma" and  *
 *     "plugin_base_menu" in the "scripting/myscripts"-  *
 *     directory of your copy of Adminmod. Place the     *
 *     files "menu.inc", "rindy.inc", "checksetting.inc",*
 *     "rindy_player_helper.inc",                        *
 *     "rindy_player_menus.inc" and                      *
 *     "rindy_player_infos.inc" in the                   *
 *     "scripting/includes"-directory of your copy of    * 
 *     Adminmod.                                         *
 *  2. Execute the "compile_all.bat" (Win32) or          *
 *     "compile_all.sh" (Linux) in the "myscripts"-      *
 *     directory.                                        *
 *  3. Grab the compiled files "plugin_base_menu.amx"    *
 *     and "plugin_bm_rindy_player.amx" from the         *
 *     "scripting/mybinaries"-directory and copy them    *
 *     into the "addons/adminmod/scripts"-directory of   *
 *     your server.                                      *
 *  4. Open the file "addons/adminmod/config/plugin.ini" *
 *     on the gameserver with a text editor and add the  *
 *     lines                                             *
 *     addons/adminmod/scripts/plugin_base_menu.amx      *
 *     addons/adminmod/scripts/plugin_bm_rindy_player.amx*
 *     to the bottom of the file. Save the changes.      *
 *  5. Open the file addons/adminmod/config/adminmod.cfg *
 *     on the gameserver and make sure to have set       *
 *     file_access_read to "1", file_access_write to "1" *
 *     and amv_enable_beta to "menu1".                   *
 *  6. Copy the files "admins.ini" and "reasons.ini"     *
 *     shipped with this plugin to the directory         *
 *     "addons/adminmod/config" of your gameserver.      *
 *  7. Set up the config files as described below.       *
 *  8. Change the map or restart your server to load     *
 *     the plugin.                                       *
 *                                                       *
 * Configuration files setup:                            *
 *  admins.ini: See comments in file for details         *
 *  reasons.ini: Place one reason per line in the file.  *
 *     All those reasons can be chosen from the menu.    *
 *     The maximum linecount is 9, and no line may       *
 *     exceed 29 characters in length.                   *
 *                                                       *
 * Vault settings:                                       *
 *   Key: PLAYER_USERSFILE                               *
 *   Default: addons/adminmod/config/admins.ini          *
 *   Description: The path to the users-file             *
 *   Key: PLAYER_LOGFILE                                 *
 *   Default: addons/adminmod/admin.log                  *
 *   Description: The path to the logfile                *
 *   Key: PLAYER_REASONFILE                              *
 *   Default: addons/adminmod/config/reasons.ini         *
 *   Description: The path to the reasons-file           *
 *   Key: PLAYER_WARNED                                  *
 *   Default: \r#                                        *
 *   Description: Tag to be displayed for warned players *
 *   Key: PLAYER_KICKED                                  *
 *   Default: \r*!*                                      *
 *   Description: Tag to be displayed for kicked players *
 *   Key: PLAYER_MEMBER                                  *
 *   Default: \y[]                                       *
 *   Description: Tag to be displayed for users with     *
 *                membertag-rights                       *
 *   Key: PLAYER_MOD                                     *
 *   Default: \y@                                        *
 *   Description: Tag to be displayed for users with     *
 *                moderator rights                       *
 *   Key: PLAYER_OP                                      *
 *   Default: \r@                                        *
 *   Description: Tag to be displayed for users with     *
 *                operator rights                        *
 *                                                       *
 * Commands:                                             *
 *  o admin_playermenu: Displays the playermenu          *
 *  o admin_warn <part of nick|#id|wonid> <reason>:      *
 *    Warns the player whose nick/id/wonid was supplied. *
 *    Optionally include a reason.                       *
 *  o admin_slap <part of nick|#id|wonid> <reason>       *
 *    Slaps the player whose nick/id/wonid was supplied. *
 *    Optionally include a reason.                       *
 *  o admin_slay <part of nick|#id|wonid> <reason>       *
 *    Warns the player whose nick/id/wonid was supplied. *
 *    Optionally include a reason.                       *
 *  o admin_kick <part of nick|#id|wonid> <reason>:      *
 *    Kicks the player whose nick/id/wonid was supplied. *
 *    Optionally include a reason.                       *
 *  o admin_ban <part of nick|#id|wonid> <time> <reason>:*
 *    Bans the player whose nick/id/wonid was supplied   *
 *    for <time> minutes. 0 minutes is a permanent ban.  *
 *    Optionally include a reason.                       *
 *  o admin_default_flags <flags>: Sets the default      *
 *    access flags. (console only)                       *
 *  o admin_reload_users: Reloads the users file         *
 *    (admins.ini). (console only)                       *
 *                                                       *
 ********************** Changelog ************************
 *                                                       *
 * Version 1.0 RC1                                       *
 *  o  Added slay command                                *
 *  o  Changed command order                             *
 *  o  Reasons are read from a config-file now           *
 *  o  Player-tags can be changed in the vault now       *
 *  o  Changed the format of usersfile to take quotes    *
 *     around the values                                 *
 *  o  Removed permanent warn and kick-flags. They are   *
 *     not used anyway                                   *
 *  o  Finally removed the unfake functionality. Will be *
 *     available in a seperate plugin                    *
 *                                                       *
 * Version 0.9.9                                         *
 *  o  Added slap command                                *
 *  o  Changed the format of usersfile and added support *
 *     for Steam-IDs.                                    *
 *  o  Added support for LAN play. When sv_lan is set to *
 *     1, the plugin uses IPs instead of Steam-IDs       *
 *                                                       *
 * Version 0.9.8                                         *
 *  o  The usersfile is parsed and stored in global      *
 *     arrays at plugin-load due to performance          *
 *  o  The data is indexed by binary-trees and a hash-   *
 *     table                                             *
 *  o  Added command admin_reload_users to reload the    *
 *     file without mapchange                            *
 *  o  Players with access to admin_unfake now are also  *
 *     informed about faking players that connect        *
 *                                                       *
 * Version 0.9.7 (development beta)                      *
 *  o  Put unfake functionality back in                  *
 *  o  Altered unfake routines to translate different    *
 *     substtrings to a username, allowing to determine  *
 *     more reliably whether a user is faked or not      *
 *  o  Added many flags                                  *
 *  o  Added command admin_default_flags to set the      *
 *     default flags (access only from console)          *
 *                                                       *
 * Version 0.9.6 (development beta)                      *
 *  o  Totally reworked user handling                    *
 *  o  Added possibility to set certain flags on a user, *
 *     allowing to discriminate between players who have *
 *     authenticated themselves to Admin-Mod and players *
 *     who haven't                                       *
 *  o  Changed format of usersfile                       *
 *  o  Changed logformat to standard:                    *
 *     L mm/dd/yyyy - hh:mm:ss: Admin "name<uid><wonid>  *
 *     <team>" kick/ban/warn [time] "name<uid><wonid>    *
 *     <team>" due to "<reason>"                         *
 *  o  Logs of command usage again go all to a single    *
 *     one file                                          *
 *  o  Made some minor changes to menues and messages    *
 *  o  Removed unfake functionality temporarily          *
 *  o  Splitted code into several include-files          *
 *  o  Completely rewrote several functions              *
 *                                                       *
 * Version 0.9.1                                         *
 *  o  Tweaked selecting the reason a bit. It now has a  *
 *     "previous" and a "next" option                    *
 *                                                       *
 * Version 0.9.0                                         *
 *  o  Fixed the bug that made only one player show in   *
 *     the admin_unfake-list again                       *
 *  o  Player recognition again tries to use passwords   *
 *                                                       *
 * Version 0.8.9                                         *
 *  o  Fixed a bug that made only one player show in the *
 *     admin_unfake-list                                 *
 *                                                       *
 * Version 0.8.8                                         *
 *  o  Added logging for warn and kick in seperate files *
 *  o  Changed bantime order                             *
 *  o  Added new Reasons and changed order               *
 *  o  Modified nicksfile to also contain clanmembers    *
 *     without admin-access, that cannot be punished     *
 *  o  Moved bantime-chosing to a seperate menu, this    *
 *     also represents a bit of a confirmation           *
 *  o  Added seperate accesslevels for all commands      *
 *  o  Greatly improved handling of disconnected players *
 *  o  Better detection of original nickname from file   *
 *  o  Added command admin_unfake, wich shows a menu     *
 *     with original names of all faked players listed   *
 *     in the nicksfile                                  *
 *  o  Changed so commands cannot be called with rcon    *
 *  o  Nickname comparison is now case-insesitive, and   *
 *     ignores clantag and spaces                        *
 *  o  Many code/CPU load optimizations                  *
 *  o  Fixed several major and minor bugs                *
 *                                                       *
 * Version 0.8.0                                         *
 *  o  Initial release                                   *
 *                                                       *
 \*******************************************************/

/* ############################################################################# *
 * ###############          YOU MAY CHANGE THESE VALUES          ############### *
 * ############################################################################# */

// Number of players that can be stored in the usersfile
#define MAX_ENTRIES 100
// Maximum string length of any configuration file
#define MAX_FILENAME_LENGTH 50
// Number of players whose data is stored after leaving
#define REMEMBER 8
// Time in seconds a recently left player will show up in the menu
// This does not affect how long their data is stored, just whether they show up in menu.
// You can still ban them after the timeout with admin_ban
#define TIMEOUT 120
// Maximum number of reasons read from reasons.ini, including hard-coded "no reason"
#define MAX_REASONS 10
// Maximum length for any one line in reasons.ini, including terminating zero
#define MAX_REASON_LENGTH 30
// The maximum length for any one tag specified in the vaultfile
#define MAX_TAG_LENGTH 10

/* ############################################################################# *
 * ###############      DO NOT CHANGE ANYTHING FROM HERE ON      ############### *
 * ############################################################################# */

/* Include Adminmod includes at first */
#include <adminlib>
#include <plugin>
/* Include custom includes */
#include <rindy>
#include <menu>
#include <checksetting>

/* Constants */
#define FLAG_MEMBER     (1<< 0) // a    Player has membertag in menu
#define FLAG_MODERATOR  (1<< 1) // b    Player has admintag in menu, and appears in moderator-list
#define FLAG_OPERATOR   (1<< 2) // c    Player is operator
#define FLAG_A_MENU     (1<< 3) // d    Player has access to menu
#define FLAG_A_WARN     (1<< 4) // e    player has access to warn
#define FLAG_A_SLAP     (1<< 5) // f    Player has access to slap
#define FLAG_A_SLAY     (1<< 6) // g    Player has access to slay
#define FLAG_A_KICK     (1<< 7) // h    Player has access to kick
#define FLAG_A_BAN      (1<< 8) // i    Player has access to ban
#define FLAG_NOTIFY     (1<< 9) // j    Player is notified about actions
#define FLAG_WARNED     (1<<24) // y    Player has warnedtag
#define FLAG_KICKED     (1<<25) // z    Player has kickedtag
#define FLAG_AUTH       (1<<26) // A    Player can authenticate with password/has an entry in users.ini
#define FLAG_IMMUNE_A   (1<<27) // B    Player is immune to non-operators
#define FLAG_IMMUNE_B   (1<<28) // C    Player is immune to everybody

#define MENU_NONE 0     // Not in menu
#define MENU_WHO 1      // In playerlist-menu (menu1)
#define MENU_WHAT 2     // In player-menu (menu2)
#define MENU_TIME 3     // In bantime-menu (menu3)
#define MENU_ACTION 4   // Performing action from menu, no menu open

#define ACTION_WARN 0
#define ACTION_SLAP 1
#define ACTION_SLAY 2
#define ACTION_KICK 3
#define ACTION_BAN 4

#define PLAYER_NOTFOUND 0
#define PLAYER_ISIMMUNE 1
#define PLAYER_ISVALID 2
#define PLAYER_NOTUNIQUE 3
#define PLAYER_FOUND 4

#define ACCESS_CONSOLE 131072
#define MAX_FLAG_LENGTH 32
#define HASHTABLESIZE (1<<4)
#define TIMES 5

/* Global Variables */
new g_Version[] = "1.0 RC1";

new g_Logfile[MAX_FILENAME_LENGTH];
new g_Usersfile[MAX_FILENAME_LENGTH];
new g_Reasonsfile[MAX_FILENAME_LENGTH];

new g_Maxplayers;
new g_Defaultflags;

new g_Reasons[MAX_REASONS][MAX_REASON_LENGTH];
new g_LoadedReasons;

new g_ShowTimes[TIMES][] = {
    "permanent",
    "1 week",
    "1 day",
    "1 hour",
    "5 minutes"
};

new g_Times[TIMES] = {
    0,
    10080,
    1440,
    60,
    5
};

new g_Warnedtag[MAX_TAG_LENGTH];
new g_Kickedtag[MAX_TAG_LENGTH];
new g_Membertag[MAX_TAG_LENGTH];
new g_Moderatortag[MAX_TAG_LENGTH];
new g_Operatortag[MAX_TAG_LENGTH];

// + 1 so we can start with [1]
new g_FiledataAuthID[MAX_ENTRIES + 1][MAX_AUTHID_LENGTH];
new g_FiledataFlags[MAX_ENTRIES + 1][2];
new g_FiledataLeft[MAX_ENTRIES + 1];
new g_FiledataRight[MAX_ENTRIES + 1];
new g_FiledataLinecount;
new g_Hashtable[HASHTABLESIZE];

new g_AuthID[MAX_PLAYERS][MAX_AUTHID_LENGTH];
new g_SessionID[MAX_PLAYERS];
new g_Flags[MAX_PLAYERS];
new g_Reason[MAX_PLAYERS];

new g_StoreAuthID[REMEMBER][MAX_AUTHID_LENGTH];
new g_StoreSessionID[REMEMBER];
new g_StoreNick[REMEMBER][MAX_NAME_LENGTH];
new g_StoreFlags[REMEMBER];
new g_StoreReason[REMEMBER];
new g_StoreTime[REMEMBER];

new g_Menu[MAX_PLAYERS];
new g_MenuStart[MAX_PLAYERS];
new g_Line[MAX_PLAYERS];
new g_VisiblePlayers[MAX_PLAYERS][7];
new g_SelectedReason[MAX_PLAYERS];
new g_SelectedPlayer[MAX_PLAYERS];
new g_Connected[MAX_PLAYERS];

/* Include helper functions from this plugin */
#include <rindy_player_helper>
#include <rindy_player_menus>
#include <rindy_player_infos>


/* Function Declarations */
forward rmb_reset(HLCommand,HLData,HLUserName,UserIndex);
forward AdminBan(HLCommand,HLData,HLUserName,UserIndex);
forward AdminKick(HLCommand,HLData,HLUserName,UserIndex);
forward AdminWarn(HLCommand,HLData,HLUserName,UserIndex);
forward Menu(HLCommand,HLData,HLUserName,UserIndex);
forward HandleMS(HLCommand,HLData,HLUserName,UserIndex);
forward do_action(UserIndex,Target[MAX_AUTHID_LENGTH],Reason[],Action,Time = 0);

/* Event Handlers */
public plugin_init() {
    new error;
    check_setting(error,"file_access_read",equal,"1");
    check_setting(error,"file_access_write",equal,"1");
    check_setting(error,"amv_enable_beta",contain,"menu1");
    check_setting(error,"admin_vault_file",fileexist);
    if(!plugin_safe_register("Rinde's Player Management Plugin","A plugin to manage admins  and, punish players",g_Version,error)) {
        return PLUGIN_CONTINUE;
    }
    plugin_registercmd("admin_warn","AdminWarn",ACCESS_ALL,"admin_warn <nick|#id|wonid> [<reason>]: Warn target.");
    plugin_registercmd("admin_slap","AdminSlap",ACCESS_ALL,"admin_slap <nick|#id|wonid> [<reason>]: Slap target.");
    plugin_registercmd("admin_slay","AdminSlay",ACCESS_ALL,"admin_slay <nick|#id|wonid> [<reason>]: Slay target.");
    plugin_registercmd("admin_kick","AdminKick",ACCESS_ALL,"admin_kick <nick|#id|wonid> [<reason>]: Kick target.");
    plugin_registercmd("admin_ban","AdminBan",ACCESS_ALL,"admin_ban <nick|#id|wonid> <time> [<reason>]: Ban target.");
    plugin_registercmd("admin_playermenu","Menu",ACCESS_ALL,"admin_playermenu: Display player menu.");
    plugin_registercmd("admin_default_flags","DefaultAccess",ACCESS_CONSOLE,"");
    plugin_registercmd("admin_reload_users","ReloadUsers",ACCESS_CONSOLE,"");
    plugin_registercmd("menuselect","HandleMS",ACCESS_ALL);
    plugin_registercmd("specmode","Connect",ACCESS_ALL);
    register_menu_reset("MenuReset");
    new str[MAX_FLAG_LENGTH];
    get_serverinfo("default_flags",str,MAX_FLAG_LENGTH);
    g_Defaultflags = read_flags(str); // Allow all possible flags, they aren't checked by command handlers if they make no sense for default flags
    vaultdata_init("PLAYER_USERSFILE",g_Usersfile,MAX_FILENAME_LENGTH,"addons/adminmod/config/admins.ini");
    vaultdata_init("PLAYER_LOGFILE",g_Logfile,MAX_FILENAME_LENGTH,"addons/adminmod/admin.log");
    vaultdata_init("PLAYER_REASONFILE",g_Reasonsfile,MAX_FILENAME_LENGTH,"addons/adminmod/config/reasons.ini");
    vaultdata_init("PLAYER_WARNED",g_Warnedtag,MAX_TAG_LENGTH,"\r#");
    vaultdata_init("PLAYER_KICKED",g_Kickedtag,MAX_TAG_LENGTH,"\r*!*");
    vaultdata_init("PLAYER_MEMBER",g_Membertag,MAX_TAG_LENGTH,"\y[]");
    vaultdata_init("PLAYER_MOD",g_Moderatortag,MAX_TAG_LENGTH,"\y@");
    vaultdata_init("PLAYER_OP",g_Operatortag,MAX_TAG_LENGTH,"\r@");
    read_users();
    read_reasons();
    get_maxplayers(g_Maxplayers);
    return PLUGIN_CONTINUE;
}

public plugin_connect(HLUserName,HLIP,UserIndex) {
    g_Connected[UserIndex] = 1;
}

public Connect(HLCommand,HLData,HLUserName,UserIndex) {
    if(!g_Connected[UserIndex]) return PLUGIN_CONTINUE;
    g_Connected[UserIndex] = 0;
    new UserName[MAX_NAME_LENGTH];
    new i;
    convert_string(HLUserName,UserName,MAX_NAME_LENGTH); //gather userdata
    get_userSessionID(UserName,g_SessionID[UserIndex]);
    if(getvar("sv_lan")) {
        get_userIP(UserName,g_AuthID[UserIndex],MAX_AUTHID_LENGTH);
    } else {
        get_userAuthID(UserName,g_AuthID[UserIndex],MAX_AUTHID_LENGTH);
    }
    g_Line[UserIndex] = get_user_line(UserIndex);
    if(((g_Flags[UserIndex] = g_FiledataFlags[g_Line[UserIndex]][0]) & FLAG_AUTH) && auth("")) { // get his flags
        g_Flags[UserIndex] = g_FiledataFlags[g_Line[UserIndex]][1] & ~FLAG_AUTH;
    }
    for(i=0;i<REMEMBER;i++) { // Search player in Store-Array
        if((g_StoreFlags[i] & (FLAG_KICKED | FLAG_WARNED)) && !strcmp(g_StoreAuthID[i],g_AuthID[UserIndex])) { //If user with matching AuthID was warned or kicked
            g_Flags[UserIndex] |= g_StoreFlags[i] & (FLAG_KICKED | FLAG_WARNED);
            if((g_Flags[UserIndex] & FLAG_KICKED)) { //If user was kicked
                new Text[MAX_TEXT_LENGTH];
                if(strcmp(UserName,g_StoreNick[i])) {  // If user has changed his name
                    snprintf(Text,MAX_TEXT_LENGTH,"(Admin) Notice: Player %s got recently kicked (%s).",UserName,g_StoreNick[i]);
                } else { // If user didn't change name
                    snprintf(Text,MAX_TEXT_LENGTH,"(Admin) Notice: Player %s got recently kicked.",UserName);
                }
                msg_admins(Text);
            }
            break;
        }
    }
    for(i=0;i<REMEMBER;i++) { //Remove user from stored users with same name if exists
        if(!strcmp(UserName,g_StoreNick[i])) {
            remove_storeddata(i);
            break; // because of this and the check in plugin_info, there can only be one with this name
        }
    }
    return PLUGIN_CONTINUE;
}

public plugin_info(HLOldName,HLNewName,UserIndex) {
    new OldName[MAX_NAME_LENGTH];
    new NewName[MAX_NAME_LENGTH];
    new i;
    convert_string(HLOldName,OldName,MAX_NAME_LENGTH);
    convert_string(HLNewName,NewName,MAX_NAME_LENGTH);
    if(strcmp(OldName,NewName) && OldName[0]) { // If player changed name
        for(i=0;i<REMEMBER;i++) { //Remove user from stored users with same name as the new name
            if(!strcmp(NewName,g_StoreNick[i])) {
                remove_storeddata(i);
                break;
            }
        }
    }
    return PLUGIN_CONTINUE;
}

public plugin_disconnect(HLUserName,UserIndex) {
    new i;
    if(g_SessionID[UserIndex]) { //When user not yet handled (ban), and fully connected
        for(i=REMEMBER-2;i>=0;i--) { //Move all stored data by one in array
            strcpy(g_StoreAuthID[i+1],g_StoreAuthID[i],MAX_AUTHID_LENGTH);
            strcpy(g_StoreNick[i+1],g_StoreNick[i],MAX_NAME_LENGTH);
            g_StoreSessionID[i+1] = g_StoreSessionID[i];
            g_StoreFlags[i+1] = g_StoreFlags[i];
            g_StoreReason[i+1] = g_StoreReason[i];
            g_StoreTime[i+1] = g_StoreTime[i];
        }
        convert_string(HLUserName,g_StoreNick[0],MAX_NAME_LENGTH); //add data of disconnecting user to first array-index
        strcpy(g_StoreAuthID[0],g_AuthID[UserIndex],MAX_AUTHID_LENGTH);
        g_StoreSessionID[0] = g_SessionID[UserIndex];
        g_StoreFlags[0] = g_Flags[UserIndex];
        g_StoreReason[0] = g_Reason[UserIndex];
        g_StoreTime[0] = systemtime();
    }
    strinit(g_AuthID[UserIndex]); //Clear global data
    g_SessionID[UserIndex] = 0;
    g_Flags[UserIndex] = 0;
    g_Reason[UserIndex] = 0;
    g_Line[UserIndex] = 0;
    return PLUGIN_CONTINUE;
}

public MenuReset(HLCommand,HLData,HLUserName,UserIndex) {
    reset_menu(HLData,g_Menu,MENU_NONE,g_Maxplayers);
    return PLUGIN_CONTINUE;
}

/* Command Handlers */
public AdminWarn(HLCommand,HLData,HLUserName,UserIndex) {
    if(!UserIndex) {
        selfmessage("This command cannot be used from console.");
        return PLUGIN_HANDLED;
    }
    if((g_Flags[UserIndex] & FLAG_AUTH) && auth("")) { // if he auth'ed
        g_Flags[UserIndex] = g_FiledataFlags[g_Line[UserIndex]][1] & ~FLAG_AUTH; // actualize flags
    }
    if(!(g_Flags[UserIndex] & FLAG_A_WARN)) {
        reject_message();
    } else {
        new Data[MAX_DATA_LENGTH];
        new TargetName[MAX_AUTHID_LENGTH];
        new Reason[70];
        convert_string(HLData,Data,MAX_DATA_LENGTH);
        strstripquotes(Data);
        strgsep(Data," ","^"",TargetName,MAX_NAME_LENGTH,Reason,70);
        if(!TargetName[0]) {
            selfmessage("Please enter a target");
        } else {
            if(strlen(Reason) == 0) {
                strcpy(Reason,g_Reasons[0],70);
            }
            do_action(UserIndex,TargetName,Reason,ACTION_WARN,0);
        }
    }
    return PLUGIN_HANDLED;
}

public AdminSlap(HLCommand,HLData,HLUserName,UserIndex) {
    if(!UserIndex) {
        selfmessage("This command cannot be used from console.");
        return PLUGIN_HANDLED;
    }
    if((g_Flags[UserIndex] & FLAG_AUTH) && auth("")) { // if he auth'ed
        g_Flags[UserIndex] = g_FiledataFlags[g_Line[UserIndex]][1] & ~FLAG_AUTH; // actualize flags
    }
    if(!(g_Flags[UserIndex] & FLAG_A_SLAP)) {
        reject_message();
    } else {
        new Data[MAX_DATA_LENGTH];
        new TargetName[MAX_AUTHID_LENGTH];
        new Reason[70];
        convert_string(HLData,Data,MAX_DATA_LENGTH);
        strstripquotes(Data);
        strgsep(Data," ","^"",TargetName,MAX_NAME_LENGTH,Reason,70);
        if(!TargetName[0]) {
            selfmessage("Please enter a target");
        } else {
            if(strlen(Reason) == 0) {
                strcpy(Reason,g_Reasons[0],70);
            }
            do_action(UserIndex,TargetName,Reason,ACTION_SLAP,0);
        }
    }
    return PLUGIN_HANDLED;
}

public AdminSlay(HLCommand,HLData,HLUserName,UserIndex) {
    if(!UserIndex) {
        selfmessage("This command cannot be used from console.");
        return PLUGIN_HANDLED;
    }
    if((g_Flags[UserIndex] & FLAG_AUTH) && auth("")) { // if he auth'ed
        g_Flags[UserIndex] = g_FiledataFlags[g_Line[UserIndex]][1] & ~FLAG_AUTH; // actualize flags
    }
    if(!(g_Flags[UserIndex] & FLAG_A_SLAY)) {
        reject_message();
    } else {
        new Data[MAX_DATA_LENGTH];
        new TargetName[MAX_AUTHID_LENGTH];
        new Reason[70];
        convert_string(HLData,Data,MAX_DATA_LENGTH);
        strstripquotes(Data);
        strgsep(Data," ","^"",TargetName,MAX_NAME_LENGTH,Reason,70);
        if(!TargetName[0]) {
            selfmessage("Please enter a target");
        } else {
            if(strlen(Reason) == 0) {
                strcpy(Reason,g_Reasons[0],70);
            }
            do_action(UserIndex,TargetName,Reason,ACTION_SLAY,0);
        }
    }
    return PLUGIN_HANDLED;
}

public AdminKick(HLCommand,HLData,HLUserName,UserIndex) {
    if(!UserIndex) {
        selfmessage("This command cannot be used from console.");
        return PLUGIN_HANDLED;
    }
    if((g_Flags[UserIndex] & FLAG_AUTH) && auth("")) { // if he auth'ed
        g_Flags[UserIndex] = g_FiledataFlags[g_Line[UserIndex]][1] & ~FLAG_AUTH; // actualize flags
    }
    if(!(g_Flags[UserIndex] & FLAG_A_KICK)) {
        reject_message();
    } else {
        new Data[MAX_DATA_LENGTH];
        new TargetName[MAX_AUTHID_LENGTH];
        new Reason[70];
        convert_string(HLData,Data,MAX_DATA_LENGTH);
        strstripquotes(Data);
        strgsep(Data," ","^"",TargetName,MAX_NAME_LENGTH,Reason,70);
        if(!TargetName[0]) {
            selfmessage("Please enter a target");
        } else {
            if(!strlen(Reason)) {
                strcpy(Reason,g_Reasons[0],70);
            }
            do_action(UserIndex,TargetName,Reason,ACTION_KICK,0);

        }
    }
    return PLUGIN_HANDLED;
}

public AdminBan(HLCommand,HLData,HLUserName,UserIndex) {
    if(!UserIndex) {
        selfmessage("This command cannot be used from console.");
        return PLUGIN_HANDLED;
    }
    if((g_Flags[UserIndex] & FLAG_AUTH) && auth("")) { // if he auth'ed
        g_Flags[UserIndex] = g_FiledataFlags[g_Line[UserIndex]][1] & ~FLAG_AUTH; // actualize flags
    }
    if(!(g_Flags[UserIndex] & FLAG_A_BAN)) {
        reject_message();
    } else {
        new Data[MAX_DATA_LENGTH];
        new TargetName[MAX_AUTHID_LENGTH];
        new sTime[MAX_NUMBER_LENGTH];
        new iTime;
        new Reason[70];
        convert_string(HLData,Data,MAX_DATA_LENGTH);
        strstripquotes(Data);
        strgsep(Data," ","^"",TargetName,MAX_NAME_LENGTH,sTime,MAX_NUMBER_LENGTH,Reason,70);
        if(!isnumeric(sTime,iTime)) {
            selfmessage("Please enter a valid ban time");
        } else if(!TargetName[0]) {
            selfmessage("Please enter a target");
        } else {
            if(!strlen(Reason)) {
                strcpy(Reason,g_Reasons[0],70);
            }
            do_action(UserIndex,TargetName,Reason,ACTION_BAN,iTime);

        }
    }
    return PLUGIN_HANDLED;
}

public Menu(HLCommand,HLData,HLUserName,UserIndex) {
    if(!UserIndex) {
        selfmessage("This command cannot be used from console.");
        return PLUGIN_HANDLED;
    }
    if((g_Flags[UserIndex] & FLAG_AUTH) && auth("")) { // if he auth'ed
        g_Flags[UserIndex] = g_FiledataFlags[g_Line[UserIndex]][1] & ~FLAG_AUTH; // actualize flags
    }
    if(!((g_Flags[UserIndex] | g_Defaultflags) & FLAG_A_MENU)) {
        reject_message();
    } else {
        kill_menu(UserIndex);
        g_MenuStart[UserIndex] = 1;
        show_menu1(UserIndex);
    }
    return PLUGIN_HANDLED;
}

public HandleMS(HLCommand,HLData,HLUserName,UserIndex) {
    if(g_Menu[UserIndex] == MENU_NONE) {
        return PLUGIN_CONTINUE;
    }
    new Data[MAX_AUTHID_LENGTH];
    new i;
    convert_string(HLData,Data,MAX_AUTHID_LENGTH);
    i = strtonum(Data);
    switch(g_Menu[UserIndex]) {
        case MENU_WHO: {
            switch(i) {
                case 8: {
                    g_MenuStart[UserIndex] -= 7;
                    show_menu1(UserIndex);
                }
                case 9: {
                    g_MenuStart[UserIndex] += 7;
                    show_menu1(UserIndex);
                }
                case 10: g_Menu[UserIndex] = MENU_NONE;
                default: {
                    g_SelectedPlayer[UserIndex] = g_VisiblePlayers[UserIndex][i-1];
                    show_menu2(UserIndex,1);
                }
            }
        }
        case MENU_WHAT: {
            switch(i) {
                case 1: {
                    g_Menu[UserIndex] = MENU_ACTION;
                    snprintf(Data,MAX_NAME_LENGTH,"#%i",g_SelectedPlayer[UserIndex]);
                    do_action(UserIndex,Data,g_Reasons[g_SelectedReason[UserIndex]],ACTION_WARN,0);
                }
                case 2: {
                    g_Menu[UserIndex] = MENU_ACTION;
                    snprintf(Data,MAX_NAME_LENGTH,"#%i",g_SelectedPlayer[UserIndex]);
                    do_action(UserIndex,Data,g_Reasons[g_SelectedReason[UserIndex]],ACTION_SLAP,0);
                    show_menu2(UserIndex,0);
                }
                case 3: {
                    g_Menu[UserIndex] = MENU_ACTION;
                    snprintf(Data,MAX_NAME_LENGTH,"#%i",g_SelectedPlayer[UserIndex]);
                    do_action(UserIndex,Data,g_Reasons[g_SelectedReason[UserIndex]],ACTION_SLAY,0);
                }
                case 4: {
                    g_Menu[UserIndex] = MENU_ACTION;
                    snprintf(Data,MAX_NAME_LENGTH,"#%i",g_SelectedPlayer[UserIndex]);
                    do_action(UserIndex,Data,g_Reasons[g_SelectedReason[UserIndex]],ACTION_KICK,0);
                }
                case 5: {
                    show_menu3(UserIndex);
                }
                case 8: {
                    if(g_SelectedReason[UserIndex] > 0) {
                        g_SelectedReason[UserIndex] -= 1;
                    } else {
                        g_SelectedReason[UserIndex] = g_LoadedReasons - 1;
                    }
                    show_menu2(UserIndex,2);
                }
                case 9: {
                    if(g_SelectedReason[UserIndex] < g_LoadedReasons - 1) {
                        g_SelectedReason[UserIndex] += 1;
                    } else {
                        g_SelectedReason[UserIndex] = 0;
                    }
                    show_menu2(UserIndex,2);
                }
                case 10:  {
                    show_menu1(UserIndex);
                }
            }
        }
        case MENU_TIME: {
            if(i == 10) {
                show_menu2(UserIndex,0);
            } else {
                g_Menu[UserIndex] = MENU_ACTION;
                snprintf(Data,MAX_NAME_LENGTH,"#%i",g_SelectedPlayer[UserIndex]);
                do_action(UserIndex,Data,g_Reasons[g_SelectedReason[UserIndex]],ACTION_BAN,g_Times[i-1]);
            }
        }
    }
    return PLUGIN_HANDLED;
}

/* Support Functions */
do_action(UserIndex,Target[MAX_AUTHID_LENGTH],Reason[],Action,Time = 0) {
    new TargetName[MAX_NAME_LENGTH];
    new TargetAuthID[MAX_AUTHID_LENGTH];
    new TargetTeam[MAX_NAME_LENGTH];
    new TargetSessionID;
    new Text[MAX_TEXT_LENGTH];
    new Commands[5][] = {"warn","slap","slay","kick","ban"};
    new present;
    new TargetIndex;
    new result = find_player(Target,present,TargetIndex);
    new Dead;
    playerinfo(TargetIndex,TargetName,MAX_NAME_LENGTH,_,_,_,Dead);
    if(g_Menu[UserIndex] == MENU_ACTION) {
        g_Menu[UserIndex] = MENU_NONE;
        if(result == PLAYER_NOTFOUND) {
            new UserName[MAX_NAME_LENGTH];
            playerinfo(UserIndex,UserName,MAX_NAME_LENGTH);
            messageex(UserName,"Player not found any more.",print_chat);
            return;
        } else {
            result = PLAYER_ISVALID;
            get_player_information(present,TargetIndex,TargetName,TargetSessionID,TargetAuthID,TargetTeam);
        }
    } else {
        switch(result) {
            case PLAYER_NOTFOUND: {
                selfmessage("Unable to find player:");
                selfmessage(Target);
                return;
            }
            case PLAYER_NOTUNIQUE: {
                selfmessage("There is more than one player matching your string:");
                selfmessage(Target);
                return;
            }
        }
        if(present) {
            if((((g_Flags[TargetIndex] | g_Defaultflags) & FLAG_IMMUNE_A) && !(g_Flags[UserIndex] & FLAG_OPERATOR)) || ((g_Flags[TargetIndex] | g_Defaultflags) & FLAG_IMMUNE_B)) {
                result = PLAYER_ISIMMUNE;
            } else {
                result = PLAYER_ISVALID;
            }
        } else {
            if((((g_StoreFlags[TargetIndex] | g_Defaultflags) & FLAG_IMMUNE_A) && !(g_Flags[UserIndex] & FLAG_OPERATOR)) || ((g_StoreFlags[TargetIndex] | g_Defaultflags) & FLAG_IMMUNE_B)) {
                result = PLAYER_ISIMMUNE;
            } else {
                result = PLAYER_ISVALID;
            }
        }
        get_player_information(present,TargetIndex,TargetName,TargetSessionID,TargetAuthID,TargetTeam);
        if(result == PLAYER_ISIMMUNE) {
            snprintf(Text,MAX_TEXT_LENGTH,"Laf. You can't %s %s. They have immunity.",Commands[Action],TargetName);
            selfmessage(Text);
            return;
        }
        if(!present && (Action == ACTION_WARN || Action == ACTION_KICK)) {
            snprintf(Text,MAX_TEXT_LENGTH,"Laf. You can't %s %s. They have left the server.",Commands[Action],TargetName);
            selfmessage(Text);
            return;
        }
        if(Action == ACTION_SLAP || Action == ACTION_SLAY) {
            if(Dead) {
                snprintf(Text,MAX_TEXT_LENGTH,"Laf. You can't %s %s. They are dead.",Commands[Action],TargetName);
                selfmessage(Text);
                return;
            }
        }
    }
    new UserName[MAX_NAME_LENGTH];
    new UserAuthID[MAX_AUTHID_LENGTH];
    new UserTeam[MAX_NAME_LENGTH];
    new UserSessionID;
    new i;
    get_player_information(1,UserIndex,UserName,UserSessionID,UserAuthID,UserTeam);
    if(Action != ACTION_BAN && Action != ACTION_KICK) {
        snprintf(Text,MAX_TEXT_LENGTH,"This is a warning from administration.^nReason: %s",Reason);
        messageex(TargetName,Text,print_console);
        messageex(TargetName,Text,print_tty);
        snprintf(Text,MAX_TEXT_LENGTH,"(Admin) Notice: %s %sed %s. Reason: %s",UserName,Commands[Action],TargetName,Reason);
        msg_admins(Text);
        strcpy(Text,Commands[Action],MAX_TEXT_LENGTH);
        if(Action == ACTION_SLAP) {
            if(!Dead) {
                slap(TargetName);
            }
        } else if(Action == ACTION_SLAY) {
            if(!Dead) {
                slay(TargetName);
            }
        }
        g_Flags[TargetIndex] |= FLAG_WARNED;
        for(i=0;i<REMEMBER;i++) {
            if(!strcmp(TargetAuthID,g_StoreAuthID[i])) {
                g_StoreFlags[i] |= FLAG_WARNED;
            }
        }
    } else if(Action == ACTION_KICK) {
        snprintf(Text,MAX_TEXT_LENGTH,"You have been kicked.^nReason: %s",Reason);
        messageex(TargetName,Text,print_console);
        kick(TargetName);
        snprintf(Text,MAX_TEXT_LENGTH,"(Admin) Notice: %s kicked %s. Reason: %s",UserName,TargetName,Reason);
        msg_admins(Text);
        strcpy(Text,Commands[Action],MAX_TEXT_LENGTH);
        g_Flags[TargetIndex] |= FLAG_KICKED;
        for(i=0;i<REMEMBER;i++) {
            if(!strcmp(TargetAuthID,g_StoreAuthID[i])) {
                g_StoreFlags[i] |= FLAG_KICKED;
            }
        }
    } else {
        if(present) {
            g_SessionID[TargetIndex] = 0; // init sessionID, userdata will not be copied to store-arrays then
            snprintf(Text,MAX_TEXT_LENGTH,"You have been banned for %i minutes.^nReason: %s",Time,Reason);
            messageex(TargetName,Text,print_console);
        } else {
            remove_storeddata(TargetIndex);
        }
        snprintf(Text,MAX_TEXT_LENGTH,"(Admin) Notice: %s banned %s for %i minutes. Reason: %s",UserName,TargetName,Time,Reason);
        msg_admins(Text);
        snprintf(Text,MAX_TEXT_LENGTH,"ban %i",Time);
        ban(TargetAuthID,Time,getvar("sv_lan") ? bBanByIP : bBanByID);
        for(i=0;i<REMEMBER;i++) {
            if(!strcmp(TargetAuthID,g_StoreAuthID[i])) {
                remove_storeddata(i);
                i--;
            }
        }
    }
    new Timestamp[MAX_NAME_LENGTH];
    servertime(Timestamp,MAX_NAME_LENGTH,"L %m/%d/%Y - %H:%M:%S");
    new Logdata[MAX_DATA_LENGTH];
    snprintf(Logdata,MAX_DATA_LENGTH,"%s: Admin ^"%s<%i><%s><%s>^" %s ^"%s<%i><%s><%s>^" due to ^"%s^"",Timestamp,UserName,UserSessionID,UserAuthID,UserTeam,Text,TargetName,TargetSessionID,TargetAuthID,TargetTeam,Reason);
    writefile(g_Logfile,Logdata,-1);
}